# Python imports
from types import DictType, FileType, StringType, UnicodeType

# Zope imports
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent

# Plone imports
from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.debug import log
from Products.Archetypes.utils import className, unique, capitalize
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import Expression, createExprContext
from Products.generator.widget import macrowidget

# Local imports
from Column import Column


class DataGridWidget(TypesWidget):
    """ Render a data table with user addable rows

        Properties:
          - columns:
            - Dictionary of "column id" : Column instance mappings
                - Column instance:
                    - title: user friendly name
                - SelectColumn instance:
                    - title: user friendly name
                    - vocabulary: vocabulary function for used to get DisplayList for dropdown    
    
    """
    
    _properties = TypesWidget._properties.copy()
    
    _properties.update({
        'macro' : "datagridwidget",
        'helper_js': ('datagridwidget.js',), 
        'columns' : {}, # Sequence of Column instances
        })
        
        
    security = ClassSecurityInfo()
        
    security.declarePublic('getColumnLabels')
    def getColumnLabels(self, field):
        """ Get user friendly names of all columns """
        
        
        if len(self.columns) == 0:
            # No explict columns renderer objects defined, 
            # just field column ids
            return field.getColumnIds()
                    
        names = []
        
        for id in field.getColumnIds():
            # Warn AT developer about his/her mistake 
            if not id in self.columns.keys():
                raise AttributeError, "DataGridWidget missing column definition for " + id + " in field " + field.getName()
            
            col = self.columns[id]
            names.append(col.getLabel())
    
        return names

    security.declarePublic('getColumnDefinition')        
    def getColumnDefinition(self, field, id):
        """ Return Column instance for column id """
        
        if id in self.columns.keys():
            return self.columns[id]        
        
        # Backward compatible/shortcut:
        # if there is explict column definition, return a dummy one
        if id in field.columns:
            return Column(id)
            
        raise KeyError, "Tried to look up missing column definition for: " + str(id)

__all__ = ('DataGridWidget')

registerWidget(DataGridWidget,
               title='Data Grid',
               description=('A data grid, like a spreadsheet'),
               used_for=('Products.DataGridField.DataGridField',)
               )

