#
# skeletonTestCase Skeleton
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.DataGridField.tests.DataGridTestCase import DataGridTestCase
from Products.Archetypes.public import *

class TestInstallation(DataGridTestCase):

    def afterSetUp(self):
        pass

    def testPortalTypes(self):
        types = self.portal.portal_types.objectIds()
        self.failUnless('DataGridDemoType' in types)

    def testPortalSkins(self):
        skins = self.portal.portal_skins.objectIds()
        self.failUnless('DataGridWidget' in skins)

    def testAddingDemo(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        self.failUnless('foo' in self.folder.objectIds())
    
    def testGettingField(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        self.failIf(self.folder.foo.getDemoField())    

    def testSettingField(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        vals = [{'column1':'joe'}]
        self.folder.foo.setDemoField(vals)
        self.assertEqual(len(self.folder.foo.getDemoField()),1)
        self.assertEqual(self.folder.foo.getDemoField()[0]['column1'],vals[0]['column1'])    

    def testSettingEmptyRows(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        vals = [{'column1':''}]
        self.folder.foo.setDemoField(vals)
        self.assertEqual(self.folder.foo.getDemoField(),())    
        self.failIf(self.folder.foo.getDemoField())
        
    def testRenderingDoesNotFail(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        self.failUnless(self.folder.foo())      
        
class TestGridAPI(DataGridTestCase):
    
    def afterSetUp(self):
        self.folder.invokeFactory('DataGridDemoType', 'demo')
        self.demo = self.folder.demo
        self.demo.setDemoField([{'column1':'a', 'column2':'b', 'The third':'c'},
                                {'column1':'d', 'column2':'e', 'The third':'f'}])
        self.field = self.demo.getField('DemoField')

    def testGetColumns(self):
        columns = self.field.getColumns(self.demo)
        self.assertEqual(columns[0], 'column1')
        self.assertEqual(columns[1], 'column2')
        self.assertEqual(columns[2], 'The third')
        
    def testGetRowCount(self):
        self.assertEqual(self.field.getRowCount(self.demo), 2)
    
    def testGetRow(self):
        self.assertEqual(self.field.getRow(self.demo, 1), {'column1':'d', 'column2':'e', 'The third':'f'})
    
    def testGetColumn(self):
        self.assertEqual(tuple(self.field.getColumn(self.demo, 'column2')), ('b', 'e',))

    def testSearchKey(self):
        self.assertEqual(self.field.search(self.demo, key='d'), ({'column1':'d', 'column2':'e', 'The third':'f'},))

    def testSearchColumn(self):
        self.assertEqual(self.field.search(self.demo, column2='e'), ({'column1':'d', 'column2':'e', 'The third':'f'},))
        
    def testSearchWithNoResults(self):
        self.assertEqual(self.field.search(self.demo, 'foo'), ())

    def testLookup(self):
        self.assertEqual(self.field.lookup(self.demo, 'd', 'column2'), 'e')
    
    def testLookupWithKeyColumn(self):
        self.assertEqual(self.field.lookup(self.demo, 'b', 'column1', 'column2'), 'a')
        
    def testLookupNotFound(self):
        self.assertEqual(self.field.lookup(self.demo, 'x', 'The third'), None)

    def testGetAsDisplayList(self):
        lst = DisplayList()
        lst.add('a', 'b')
        lst.add('d', 'e')
        self.assertEqual(self.field.getAsDisplayList(self.demo), lst)
        
    def testGetAsDisplayListWithCustomColumns(self):
        lst = DisplayList()
        lst.add('c', 'a')
        lst.add('f', 'd')
        self.assertEqual(self.field.getAsDisplayList(self.demo, 'The third', 'column1'), lst)
        
    def testGetAsGrid(self):
        self.assertEqual(tuple(self.field.getAsGrid(self.demo)), (('a', 'b', 'c',),
                                                                  ('d', 'e', 'f'),))

    def testQueryDataGrid(self):
        self.assertEqual(self.demo.queryDataGrid('DemoField', 'lookup', ['d', 'column2']), 'e')
        
    def testSearch(self):
        catalog_tool = self.portal.portal_catalog
        # TODO: code goes here
        
        
                                                                    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestGridAPI))
    return suite

if __name__ == '__main__':
    framework()
